@@ -1,7 +1,7 @@
 #' EQF plot
 #'
-#' Depict the EQF plot by the result of permutation process to detect
-#' the QTL hotspot.
+#' Depict the EQF plot by the result of the permutation process to
+#' detect the QTL hotspot.
 #'
 #' @param result list. The data list of the output from LOD.QTLdetect()
 #' or EQF.permu().
@@ -70,16 +70,22 @@ EQF.plot <- function(result, plot.all = TRUE, plot.chr = TRUE){
   }
   eqf.all <- apply(EQF, 2, sum)
 
-  x0 <- 1:length(cr0)+(cr0-1)*100
-  xn <- ncr+(0:(nc-1))*100-lcr/2
+  gate <- ceiling(length(cr0)/nc/2)
+  x0 <- 1:length(cr0)+(cr0-1)*gate
+  xn <- ncr+(0:(nc-1))*gate-lcr/2
 
   oldpar <- graphics::par(no.readonly = TRUE)
   on.exit(graphics::par(oldpar))
 
+  yli <- max(eqf.all)*1.2
+  xli <- max(x0)/40
+  yax <- 2
+  if(yli > 20){
+    yax <- 5
+  }
   if(plot.all & nc>1){
     graphics::par(mfrow = c(1, 1))
     graphics::par(mai = c(1, 1, 1, 1))
-    yli <- max(eqf.all)*1.2
     ma <- paste("LOD thresholds =", thre)
     if(length(clumatrix) > 0){
       ma <- paste("LOD thresholds =", thre, "  # of group =", nrow(clumatrix))
@@ -90,16 +96,17 @@ EQF.plot <- function(result, plot.all = TRUE, plot.chr = TRUE){
     graphics::axis(side = 1, pos = -yli/35, at = xn, labels = 1:nc, cex.axis = 1.2, tick = FALSE)
     lse <- 4000/max(x0)
     graphics::segments(x0, rep(-yli/35, length(x0)), x0, rep(-yli/70, length(x0)), lwd = lse)
-    graphics::segments(-100, 0, -100, yli)
-    graphics::segments(-100, yli, max(x0)+100, yli)
-    graphics::segments(max(x0)+100, 0, max(x0)+100, yli)
+    graphics::segments(-xli, -yli/35, max(x0)+xli, -yli/35)
+    graphics::segments(-xli, -yli/35, -xli, yli)
+    graphics::segments(-xli, yli, max(x0)+xli, yli)
+    graphics::segments(max(x0)+xli, -yli/35, max(x0)+xli, yli)
     if(length(eqfthre) > 0){
       for(j in 1:nrow(eqfthre)){
         graphics::axis(2, eqfthre[j, 1], las = 2)
         graphics::axis(4, paste(rownames(eqfthre)[j], " (", eqfthre[j, 2], ")", sep = ""), at = eqfthre[j], las = 2)
       }
       graphics::abline(h = eqfthre[, 1], col = "red")
-    } else {graphics::axis(2, seq(0, yli, 20), las = 2)}
+    } else {graphics::axis(2, seq(0, yli, 2), las = 2)}
   }
 
   if(plot.chr | (nc == 1 & plot.all)){
@@ -124,7 +131,7 @@ EQF.plot <- function(result, plot.all = TRUE, plot.chr = TRUE){
           graphics::axis(4, paste(rownames(eqfthre)[k], " (", eqfthre[k, 2], ")", sep = ""), at = eqfthre[k], las = 2.5)
         }
         graphics::abline(h = eqfthre[, 1], col = "red")
-      } else {graphics::axis(2, seq(0, yli, 20), las = 2.5)}
+      } else {graphics::axis(2, seq(0, yli, 2), las = 2.5)}
     }
   }
 }
